/* glcd_functions.cpp -----------------------------------------------------------------------*/

#include <Wire.h>
#include <avr/pgmspace.h>
#include <glcd_functions_avr.h>

// Table of graphic instruction
#define GLCD_IGNORE 0
#define GLCD_ON 1
#define GLCD_OFF 2
#define GLCD_SET_LIGHT 3
#define GLCD_DIM_LIGHT_ON 4
#define GLCD_DIM_LIGHT_OFF 5
#define GLCD_SET_I2C 6
#define GLCD_VERBOSE_ON 7
#define GLCD_VERBOSE_OFF 8
#define GLCD_SET_AUTO_SCROLL_ON 9
#define GLCD_SET_AUTO_SCROLL_OFF 10
#define GLCD_CLEAR_SCREEN 15
#define GLCD_SET_PIXEL 16
#define GLCD_DRAW_LINE 17
#define GLCD_DRAW_DOTTED_VER_LINE 19
#define GLCD_DRAW_DOTTED_HOR_LINE 20
#define GLCD_DRAW_FUNCTION 21
#define GLCD_DRAW_SCATTER 22
#define GLCD_DRAW_RECTANGLE 23
#define GLCD_DRAW_FILLED_RECTANGLE 24
#define GLCD_DRAW_CIRCLE 25
#define GLCD_DRAW_FILLED_CIRCLE 26
#define GLCD_LOAD_RAW 30
#define GLCD_DRAW_CHAR 32
#define GLCD_DRAW_STR 33
#define GLCD_DRAW_CENTER_STR 34
#define GLCD_DRAW_UDEC 35
#define GLCD_DRAW_SDEC 36
#define GLCD_DRAW_RADJ_STR 37
#define GLCD_SET_CURSOR 50
#define GLCD_SET_FONT 51
#define GLCD_SCROLL_UP 52
#define GLCD_GET_STRING_WIDTH 60
#define GLCD_GET_CURSOR 61
#define GLCD_GET_FONT 62
#define GLCD_GET_MAX_XY 63
#define GLCD_GET_FONT_HEIGHT 65


glcd::glcd(uint8_t address) {
	glcd_address = address;
}

void glcd::glcd_on(void) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_ON);			  
	Wire.endTransmission();
}

void glcd::glcd_off(void) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_OFF);			  
	Wire.endTransmission();
}

void glcd::clear_screen(void) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_CLEAR_SCREEN);  // send token for "clear screen"
  Wire.write(0);                  // set pixels to "off"
  Wire.endTransmission();         
}

void glcd::set_light(uint8_t light_level) {
  Wire.beginTransmission(glcd_address);	
  Wire.write(GLCD_SET_LIGHT);		// send token for "clear screen"
  Wire.write(light_level);          // new light level 0 ... 10
  Wire.endTransmission();		
}

void glcd::set_pixel(uint8_t x, uint8_t y, uint8_t mode) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_SET_PIXEL);     // send command token 
  Wire.write(x);                  // send coordinate
  Wire.write(y);                  // send coordinate
  Wire.write(mode);                 // send drawing mode
  Wire.endTransmission();         
}

void glcd::draw_udec(uint16_t number, uint8_t spaces, uint8_t decimal_places) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_DRAW_UDEC);      // send command token 
  Wire.write(number >> 8);        // send high byte
  Wire.write(number & 0x00FF);    // send low byte
  Wire.write(spaces);             // send number of spaces
  Wire.write(decimal_places);     // send number of digits aftrer decimal point           
  Wire.endTransmission();         
}

void glcd::draw_sdec(uint16_t number, uint8_t spaces, uint8_t decimal_places) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_SDEC);      // send command token
	Wire.write(number >> 8);        // send high byte
	Wire.write(number & 0x00FF);    // send low byte
	Wire.write(spaces);             // send number of spaces
	Wire.write(decimal_places);     // send number of digits aftrer decimal point
	Wire.endTransmission();
}

void glcd::draw_char(char c) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_DRAW_CHAR);     // send command token 
  Wire.write(c);                  // send character
  Wire.endTransmission();         
}

void glcd::set_cursor(uint8_t x, uint8_t y) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_SET_CURSOR);    // send command token 
  Wire.write(x);                  // send cursor data
  Wire.write(y);                  // send cursor data
  Wire.endTransmission();         
}

void glcd::draw_rectangle(uint8_t x_start, uint8_t y_start, uint8_t x_end, uint8_t y_end, uint8_t mode) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_DRAW_RECTANGLE);   // send command token 
  Wire.write(x_start);               // send rectangle data
  Wire.write(y_start);               // send rectangle data
  Wire.write(x_end);                 // send rectangle data
  Wire.write(y_end);                 // send rectangle data
  Wire.write(mode);                 // send drawing mode
  Wire.endTransmission();         
}

void glcd::draw_filled_rectangle(uint8_t x_start, uint8_t y_start, uint8_t x_end, uint8_t y_end, uint8_t mode) {
  Wire.beginTransmission(glcd_address); 
  Wire.write(GLCD_DRAW_FILLED_RECTANGLE);   // send command token 
  Wire.write(x_start);               // send rectangle data
  Wire.write(y_start);               // send rectangle data
  Wire.write(x_end);                 // send rectangle data
  Wire.write(y_end);                 // send rectangle data
  Wire.write(mode);					 // send drawing mode
  Wire.endTransmission();            
}

void glcd::dim_on(void) {
  Wire.beginTransmission(glcd_address);  
  Wire.write(GLCD_DIM_LIGHT_ON);     // send command token 
  Wire.endTransmission();         
}

void glcd::dim_off(void) {
  Wire.beginTransmission(glcd_address);  
  Wire.write(GLCD_DIM_LIGHT_OFF);     			// send command token 
  Wire.endTransmission();         
}

void glcd::draw_str(const char my_str[]) {
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
    
  Wire.beginTransmission(glcd_address);			// send command token
  Wire.write(GLCD_DRAW_STR);
  do {
	my_byte = my_str[i];
	Wire.write(my_byte);
	++i;
	if (++j == 30) {
      Wire.endTransmission();
	  delay(1);
	  j = 0;
      Wire.beginTransmission(glcd_address);		// send command token
	};
  } while (my_byte > 0);
  Wire.write(0);                      			// send termination code
  Wire.endTransmission();
 }
 
void glcd::draw_pgm_str(const char my_str[])
{
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
  
  Wire.beginTransmission(glcd_address);			// send command token
  Wire.write(GLCD_DRAW_STR);
  do {
	my_byte = pgm_read_byte(my_str + i);		// get data byte
	Wire.write(my_byte);						// send data bytes
	++i;
	if (++j == 30) {
	  Wire.endTransmission();
	  delay(1);
	  j = 0;
	  Wire.beginTransmission(glcd_address);
    };	  
  } while (my_byte > 0);		      			// send termination code
  Wire.endTransmission();         
}

void glcd::draw_center_str(const char my_str[]) {
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
  
  Wire.beginTransmission(glcd_address);  
  Wire.write(GLCD_DRAW_CENTER_STR);   			// send command token 
  do {
	  my_byte = my_str[i];
	  Wire.write(my_byte);						// send data bytes
	  ++i;
	  if (++j == 30) {
		  Wire.endTransmission();
		  delay(1);
		  j = 0;
		  Wire.beginTransmission(glcd_address);

	  };	  
  } while (my_byte > 0);		      			// send termination code
  Wire.endTransmission();         
}

void glcd::draw_center_pgm_str(const char my_str[]) {
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
  
  Wire.beginTransmission(glcd_address);			// send command token
  Wire.write(GLCD_DRAW_CENTER_STR);
  do {
	my_byte = pgm_read_byte(my_str + i);		// get data byte
	Wire.write(my_byte);						// send data bytes
	++i;
	if (++j == 30) {
	  Wire.endTransmission();
	  delay(1);
	  j = 0;
	  Wire.beginTransmission(glcd_address);
    };	  
  } while (my_byte > 0);		      			// send termination code
  Wire.endTransmission();         
}	

void glcd::draw_radj_str(char my_str[]) {
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
  
  Wire.beginTransmission(glcd_address);			// send command token
  Wire.write(GLCD_DRAW_RADJ_STR);
  do {
	my_byte = my_str[i];						// get data byte
	Wire.write(my_byte);						// send data bytes
	++i;
	if (++j == 30) {
	  Wire.endTransmission();
	  delay(1);
	  j = 0;
	  Wire.beginTransmission(glcd_address);
    };	  
  } while (my_byte > 0);		      			// send termination code
  Wire.endTransmission();         
}

void glcd::draw_radj_pgm_str(char my_str[]) {
  uint8_t i = 0, j = 0;
  uint8_t my_byte;
  
  Wire.beginTransmission(glcd_address);			// send command token
  Wire.write(GLCD_DRAW_RADJ_STR);
  do {
	my_byte = pgm_read_byte(my_str + i);		// get data byte
	Wire.write(my_byte);						// send data bytes
	++i;
	if (++j == 30) {
	  Wire.endTransmission();
	  delay(1);
	  j = 0;
	  Wire.beginTransmission(glcd_address);
    };	  
  } while (my_byte > 0);		      			// send termination code
  Wire.endTransmission();         
}

void glcd::set_font(uint8_t font) {
  Wire.beginTransmission(glcd_address);  
  Wire.write(GLCD_SET_FONT);      // send command token 
  Wire.write(font);               // send font number 
  Wire.endTransmission();         
}

void glcd::draw_line(uint8_t x_start, uint8_t y_start, uint8_t x_end, uint8_t y_end, uint8_t mode) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_LINE);			// send command token
	Wire.write(x_start);				// send coordinates
	Wire.write(y_start);				// send coordinates
	Wire.write(x_end);					// send coordinates
	Wire.write(y_end);					// send coordinates
	Wire.write(mode);					// send drawing mode
	Wire.endTransmission();
}

void glcd::draw_dotted_ver_line(uint8_t x_pos, uint8_t y_start, uint8_t y_end, uint8_t spaces, uint8_t mode) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_DOTTED_VER_LINE);   
	Wire.write(x_pos);               
	Wire.write(y_start);               
	Wire.write(y_end);                 
	Wire.write(spaces);
	Wire.write(mode);					 
	Wire.endTransmission();
}

void glcd::draw_dotted_hor_line(uint8_t y_pos, uint8_t x_start, uint8_t x_end, uint8_t spaces, uint8_t mode) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_DOTTED_HOR_LINE);
	Wire.write(y_pos);
	Wire.write(x_start);
	Wire.write(x_end);
	Wire.write(spaces);
	Wire.write(mode);
	Wire.endTransmission();
}

void glcd::draw_circle(uint8_t x_org, uint8_t y_org, uint8_t radius, uint8_t segment, uint8_t mode) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_CIRCLE);
	Wire.write(x_org);
	Wire.write(y_org);
	Wire.write(radius);
	Wire.write(segment);
	Wire.write(mode);
	Wire.endTransmission();
}

void glcd::draw_filled_circle(uint8_t x_org, uint8_t y_org, uint8_t radius, uint8_t segment, uint8_t mode) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_FILLED_CIRCLE);
	Wire.write(x_org);
	Wire.write(y_org);
	Wire.write(radius);
	Wire.write(segment);
	Wire.write(mode);
	Wire.endTransmission();
}

void glcd::draw_function(const uint8_t count, const uint8_t x_pos, uint8_t mode, uint8_t values[]) {
	uint8_t val_start, val_end, stop_flag;
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_FUNCTION);
	Wire.write(count);
	Wire.write(x_pos);
	Wire.write(mode);
	Wire.endTransmission();

	// data transmission is broken into individual packages of 30 bytes
	val_start = 0;
	do {
		if (count > val_start + 30) {
			val_end = val_start + 30;
			stop_flag = 0;
		} else {
			val_end = count;
			stop_flag = 1;
		};
		Wire.beginTransmission(glcd_address);
		Wire.write(&values[val_start], val_end - val_start);
		Wire.endTransmission();
		val_start += 30;
	} while (stop_flag == 0);
}

void glcd::draw_scatter(const uint8_t count, const uint8_t x_pos, uint8_t mode, uint8_t values[]) {
	uint8_t val_start, val_end, stop_flag;
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_DRAW_SCATTER);
	Wire.write(count);
	Wire.write(x_pos);
	Wire.write(mode);
	Wire.endTransmission();

	// data transmission is broken into individual packages of 30 bytes
	val_start = 0;
	do {
		if (count > val_start + 30) {
			val_end = val_start + 30;
			stop_flag = 0;
		} else {
			val_end = count;
			stop_flag = 1;
		};
		Wire.beginTransmission(glcd_address);
		Wire.write(&values[val_start], val_end - val_start);
		Wire.endTransmission();
		val_start += 30;
	} while (stop_flag == 0);
}

void glcd::auto_scroll_on(void) {
  Wire.beginTransmission(glcd_address);
  Wire.write(GLCD_SET_AUTO_SCROLL_ON);
  Wire.endTransmission();
};

void glcd::auto_scroll_off(void) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_SET_AUTO_SCROLL_OFF);
	Wire.endTransmission();
};

uint8_t glcd::get_str_width(char str[]) {
	uint8_t str_len, my_char;
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_GET_STRING_WIDTH);
	do {
		my_char = *str;
		Wire.write(my_char);
		++str;
	} while (my_char > 0);
	Wire.endTransmission();
	delay(5);
	Wire.requestFrom(glcd_address, 1, true);
	while (Wire.available())
		str_len = Wire.read();	
	Wire.endTransmission();
	return(str_len);
}

void glcd::get_max_xy(uint8_t *x_max, uint8_t *y_max) {
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_GET_MAX_XY);
	Wire.endTransmission();
	Wire.requestFrom(glcd_address, 2, true);
	while (Wire.available()) { 
		*x_max = Wire.read();
		*y_max = Wire.read();
	};
	Wire.endTransmission();
}

uint8_t glcd::get_font(void) {
	uint8_t font;
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_GET_FONT);
	Wire.endTransmission();
	Wire.requestFrom(glcd_address, 1, true);
	while (Wire.available())
		font = Wire.read();
	return(font);
}

void glcd::get_cursor(uint8_t *x_cur, uint8_t *y_cur) {
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_GET_CURSOR);
	Wire.endTransmission();
	Wire.requestFrom(glcd_address, 2, true);
	while (Wire.available()) {
		*x_cur = Wire.read();
		*y_cur = Wire.read();
	};
}

uint8_t glcd::get_font_height(void) {
	uint8_t font_height;
	
	Wire.beginTransmission(glcd_address);
	Wire.write(GLCD_GET_FONT_HEIGHT);
	Wire.endTransmission();
	Wire.requestFrom(glcd_address, 1, true);
	while (Wire.available())
		font_height = Wire.read();
	return(font_height);
}


